<?php

declare(strict_types=1);

namespace RoteringCore;

 /* This script was written by Thorsten Rotering, 2023.
  *
  * This script is free software: you can redistribute it and/or modify
  * it under the terms of the Lesser GNU General Public License (LGPL) as
  * published by the Free Software Foundation, either version 3 of the
  * License or (at your option) any later version.
  *
  * This script is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License for more details.
  *
  * You should received a copy of the GNU General Public License and the
  * Lesser GNU General Public License with this script. If not, see
  * <http://www.gnu.org/licenses/>. */

class RuntimeMeter
{
    const VERSION = '1.1.0';
    const PUBLIC_NAME = 'RuntimeMeter';
    const INTERNAL_NAME = 'Klepsydra';

    const FORMAT_UNIT = 0;
    const FORMAT_SHOW_UNIT_SYMBOL = 1;
    const FORMAT_DECIMALS = 2;
    const FORMAT_DECIMAL_SEPARATOR = 3;
    const FORMAT_THOUSANDS_SEPARATOR = 4;

    const UNIT_MICROSECONDS = -2;
    const UNIT_MILLISECONDS = -1;
    const UNIT_SECONDS = 0;

    const SEPARATOR_NONE = '';
    const SEPARATOR_DOT = '.';
    const SEPARATOR_COMMA = ',';
    const SEPARATOR_SPACE = '&nbsp;';
    const SEPARATOR_THINSPACE = '&#8239;';
    const SEPARATOR_APOSTROPHE = '&rsquo;';

    /**
     * The time the measurement was started as UNIX timestamp in microseconds.
     * @var float
     */
    private float $aStartTime = 0.0;
    
    /**
     * The runtime of the last measurement in microseconds.
     * @var float
     */
    private float $aRunTime = 0.0;
    
    /**
     * Indicates wether the measurement is running or not.
     * @var bool
     */
    private bool $aIsRunning = false;

    /**
     * The unit of the formatted result string.
     * @var int
     */
    private int $aFormatUnit = self::UNIT_MILLISECONDS;
    
    /**
     * Indicates wheter the formatted result string should contain the unit symbol or not.
     * @var bool
     */
    private bool $aFormatShowUnitSymbol = true;
    
    /**
     * The number of decimals in the formatted result string.
     * @var int
     */
    private int $aFormatDecimals = 3;
    
    /**
     * The separator string that separates 
     * @var string
     */
    private string $aFormatDecimalSeparator = self::SEPARATOR_COMMA;
    
    /**
     * The 
     * @var string
     */
    private string $aFormatThousandsSeparator = self::SEPARATOR_SPACE;


    /* Creates a new object of the RuntimeMeter. */
    public function __construct()
    {
    }

    /**
     * Starts the measurement.
     * @param bool $pOverride By default, any subsequent calls of this method will be ignored, while the measurement is already running. This behaviour can be
     * overriden by setting this parameter to true. In this case calling this method will always restart the measurement.
     * @return bool True, if the measurement was started successfully. If you call this method while the measurement is already running and you make no use
     * of the override option, false will be returned.
     */
    public function start(bool $pOverride = false): bool
    {
        if ($pOverride || !$this->isRunning()) {
            $this->setStartTime(microtime(true));
            $this->aIsRunning = true;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Stops the measurement.
     * @return bool True, if the measurement was stopped successfully. If you call this method while the measurement is not running, false will be returned.
     */
    public function stop(): bool
    {
        if ($this->isRunning()) {
            $this->aIsRunning = false;
            $this->setRunTime(microtime(true) - $this->getStartTime() + $this->getRunTime());
            return true;
        } else {
            return false;
        }
    }

    /**
     * Resets the measurement.
     */
    public function reset()
    {
        $lDoRestart = $this->stop();
        $this->setRunTime(0.0);
        
        if ($lDoRestart) {
            $this->start();
        }
    }

    /**
     * Returns, wether the measurement is currently running.
     * @return bool True, if the measurement is currently running, otherwise false.
     */
    public function isRunning(): bool
    {
        return $this->aIsRunning;
    }

    /**
     * Returns the time the measurement was started.
     * @return float The time as a UNIX timestamp in microseconds.
     */
    public function getStartTime(): float
    {
        return $this->aStartTime;
    }

    /**
     * Sets the time the measurement was started.
     * @param float $pTime The time as a UNIX timestamp in microseconds.
     */
    private function setStartTime(float $pTime)
    {
        $this->aStartTime = max($pTime, 0.0);
    }

    /**
     * Returns the result of the last measurement as formatted string.
     * @return string The result as formatted string.
     */
    public function getResult(): string
    {
        $lRunTimeRaw = $this->getRunTime();

        switch ($this->aFormatUnit) {
            
            case self::UNIT_MICROSECONDS:
                $lUnitSymbol = 'µs'; 
                $lRunTime = $lRunTimeRaw * 1000000;
                break;
            
            case self::UNIT_MILLISECONDS:
                $lUnitSymbol = 'ms';
                $lRunTime = $lRunTimeRaw * 1000;
                break;
            
            case self::UNIT_SECONDS:
                $lUnitSymbol = 's';
                $lRunTime = $lRunTimeRaw;
                break;
            
            default:
                $lUnitSymbol = '';
        }

        $lResultStep1 = number_format($lRunTime, $this->aFormatDecimals, '<', '>');
        $lResultStep2 = str_replace('<', $this->aFormatDecimalSeparator, $lResultStep1);
        $lResultStep3 = str_replace('>', $this->aFormatThousandsSeparator, $lResultStep2);
        
        if ($this->aFormatShowUnitSymbol) {
            return $lResultStep3 . '&nbsp;' . $lUnitSymbol;
        } else {
            return $lResultStep3;
        }
    }

    /**
     * Returns the runtime of the measurement.
     * @return float The runtime in microseconds.
     */
    public function getRunTime(): float
    {
        if ($this->isRunning()) {
            $this->stop();
            $this->start();
        }
        
        return $this->aRunTime;
    }

    /**
     * Sets the runtime.
     * @param float $pTime The time as UNIX timestamp in microseconds.
     */
    private function setRunTime(float $pTime)
    {
        $this->aRunTime = max($pTime, 0);
    }

    /**
     * Sets an attributes of the formatted result string.
     * @param int $pAttribute The attribute to set. Use the FORMAT_* constants.
     * @param mixed $pValue The value to set the attribute to.
     */
    public function setResultFormat(int $pAttribute, mixed $pValue)
    {
        switch ($pAttribute) {
            case self::FORMAT_UNIT:
                $this->aFormatUnit = (int)$pValue;
                break;
            case self::FORMAT_SHOW_UNIT_SYMBOL:
                $this->aFormatShowUnitSymbol = ($pValue == true);
                break;
            case self::FORMAT_DECIMALS: 
                $this->aFormatDecimals = max((int)$pValue, 0);
                break;
            case self::FORMAT_DECIMAL_SEPARATOR:
                $this->aFormatDecimalSeparator =(string)$pValue;
                break;
            case self::FORMAT_THOUSANDS_SEPARATOR:
                $this->aFormatThousandsSeparator = (string)$pValue;
                break;
        }
    }
 }
 